<?php

// Add the "Settings" link for PeachPay in the list of installed plugins
function peachpay_add_settings_link( $links ) {
    $settings_link = '<a href="admin.php?page=peachpay">' . __( 'Settings' ) . '</a>';
    array_unshift( $links, $settings_link );
    return $links;
}

function peachpay_settings_general() {
    add_settings_section(
        'peachpay_section_general',
        __( 'General', 'peachpay' ),
        null,
        'peachpay'
    );
    add_settings_field(
        'peachpay_field_key', // as of WP 4.6 this value is used only internally
        // use $args' label_for to populate the id inside the callback
        __( 'Activation key', 'peachpay' ),
        'peachpay_field_key_cb',
        'peachpay',
        'peachpay_section_general',
        [ 'label_for' => 'peachpay_field_key' ]
    );
    add_settings_field(
        'peachpay_field_language',
        __( 'Language', 'peachpay' ),
        'peachpay_field_language_cb',
        'peachpay',
        'peachpay_section_general',
        [ 'label_for' => 'peachpay_field_language' ]
    );
    add_settings_field(
        'peachpay_field_test_mode',
        __( 'Test mode', 'peachpay' ),
        'peachpay_field_test_mode_cb',
        'peachpay',
        'peachpay_section_general',
        [ 'label_for' => 'peachpay_field_test_mode' ]
    );
    add_settings_field(
        'peachpay_field_file_upload',
        __( 'File upload', 'peachpay' ),
        'peachpay_field_file_upload_cb',
        'peachpay',
        'peachpay_section_general',
        [ 'label_for' => 'peachpay_field_file_upload' ]
    );
    add_settings_field(
        'peachpay_field_email_receipt',
        __( 'Email receipt', 'peachpay' ),
        'peachpay_field_email_receipt_cb',
        'peachpay',
        'peachpay_section_general',
        [ 'label_for' => 'peachpay_field_email_receipt' ]
    );
}

/**
 * Key field callback function
 *
 * WordPress has magic interaction with the following keys: label_for, class.
 * - the "label_for" key value is used for the "for" attribute of the <label>.
 * - the "class" key value is used for the "class" attribute of the <tr> containing the field.
 * Note: you can add custom key value pairs to be used inside your callbacks.
 *
 * @param array $args
 */
function peachpay_field_key_cb( $args ) {
    $options = get_option( 'peachpay_options' );
    ?>
    <input
        id='peachpay_setting_key'
        name='peachpay_options[key]'
        type='text'
        value='<?php echo esc_attr( $options ? $options['key'] : '' ); ?>'
        style='width: 300px'
    />
    <p class="description">If you activated PeachPay recently, we'll be sending you the activation key soon! Didn't hear from us or need help? Send an email: <a href="mailto:info@peachpay.app">info@peachpay.app</a></p>
    <?php
}

function peachpay_field_language_cb( $args ) {
    $options = get_option( 'peachpay_options' );
    $languages = [
        'English (US)' => 'en-US',
        'Español' => 'es-ES',
        'Français' => 'fr',
        'Italiano' => 'it',
        'Deutsch' => 'de-DE',
    ];
    ?>
    <select
        id="peachpay_setting_language"
        name="peachpay_options[language]">
        <?php foreach ( $languages as $language => $value ) { ?>
            <option
                value="<?php echo $value ?>"
                <?php echo isset( $options[ 'language' ] ) ? ( selected( $options[ 'language' ], $value, false ) ) : ( '' ); ?>
            >
                <?php echo $language ?>
            </option>
        <?php } ?>
    </select>
    <p class="description">This will change the language on the button and in the checkout flow.</p>
    <?php
}

function peachpay_field_file_upload_cb( $args ) {
    $options = get_option( 'peachpay_options' );
    ?>
    <input
        type="checkbox"
        name="peachpay_options[file_upload]"
        id="peachpay_options[file_upload]"
        value="1"
        <?php checked(1, $options['file_upload'] ?? false, true); ?>
    >
    <label for="peachpay_options[file_upload]">Enable file uploads</label>
    <p class="description">Give customers the option to upload files as part of the checkout. A link to the uploaded file can be found in the customer's order note. Currently only supports image files.</p>
    <?php
}

function peachpay_field_test_mode_cb( $args ) {
    $options = get_option( 'peachpay_options' );
    ?>
    <input
        type="checkbox"
        name="peachpay_options[test_mode]"
        id="peachpay_options[test_mode]"
        value="1"
        <?php checked(1, $options['test_mode'] ?? false, true); ?>
    >
    <label for="peachpay_options[test_mode]">Enable test mode</label>
    <p class="description">Test mode allows you to make purchases using the <b>4242 4242 4242 4242</b> test card, with expiration <b>04/24</b> and CVC <b>444</b>. This helps you see how PeachPay works on your store. You do not need to connect a Stripe account to use test mode.
    <?php
}

function peachpay_field_email_receipt_cb( $args ) {
    $options = get_option( 'peachpay_options' );
    ?>
    <input
        type="checkbox"
        name="peachpay_options[email_receipt]"
        id="peachpay_options[email_receipt]"
        value="1"
        <?php checked(1, $options['email_receipt'] ?? false, true); ?>
    >
    <label for="peachpay_options[email_receipt]">Send PeachPay email receipt</label>
    <p class="description">PeachPay will send an <strong>additional</strong> email receipt if this setting is turned on. If you already have email receipts configured, you probably don't need this. Turning on this setting will not affect existing email receipt settings in any way.</p>
    <?php
}

function peachpay_settings_button() {
    add_settings_section(
        'peachpay_section_button',
        __( 'Button preferences', 'peachpay' ),
        null,
        'peachpay'
    );
    add_settings_field(
        'peachpay_field_button_color',
        __( 'Button color', 'peachpay' ),
        'peachpay_field_button_color_cb',
        'peachpay',
        'peachpay_section_button',
        [ 'label_for' => 'peachpay_field_button_color' ]
    );
    foreach ( [ 'product_button_position', 'cart_button_position' ] as $key ) {
        $page = $key === 'cart_button_position' ? 'cart' : 'product';
        add_settings_field(
            "peachpay_field_$key",
            __( "Button position ($page page)", 'peachpay' ),
            'peachpay_field_button_position_cb',
            'peachpay',
            'peachpay_section_button',
            [ 'key' => $key ]
        );
    }
    foreach ( [ 'button_width_product_page', 'button_width_cart_page' ] as $key ) {
        $page = $key === 'button_width_cart_page' ? 'cart' : 'product';
        add_settings_field(
            "peachpay_field_$key",
            __( "Button width ($page page)", 'peachpay' ),
            "peachpay_field_button_width_cb",
            'peachpay',
            'peachpay_section_button',
            [ 'key' => $key ]
        );
    }
}

function peachpay_field_button_color_cb( $args ) {
    $options = get_option( 'peachpay_options' );
    ?>
    <input
        id='peachpay_setting_button_color_custom'
        name='peachpay_options[button_color]'
        type='color'
        value='<?php echo esc_attr( $options ? $options[ 'button_color' ] : '#ffa500' ); ?>'
        style='width: 75px'
        list='presets'
    />
    <datalist id="presets">
        <option>#ffa500</option>
        <option>#ff8ba8</option>
        <option>#ff4d39</option>
        <option>#5cab5e</option>
        <option>#0286e7</option>
        <option>#af57ec</option>
        <option>#111111</option>
    </datalist>
    <?php
}

function peachpay_field_button_position_cb( $args ) {
    $options = get_option( 'peachpay_options' );
    $key = $args[ 'key' ];
    $positions_product_page = [
        'Left' => 'left',
        'Right' => 'right',
        'Full' => 'full',
    ];
    $positions_cart_page = [
        'Full' => 'full',
        'Left' => 'left',
        'Right' => 'right',
    ];
    $positions = $key === 'cart_button_position' ? $positions_cart_page : $positions_product_page;
    ?>
    <select
        id="<?php echo esc_attr( $key ); ?>"
        name="peachpay_options[<?php echo esc_attr( $key ); ?>]">
        <?php foreach ( $positions as $position => $value ) { ?>
            <option
                value="<?php echo $value ?>"
                <?php echo isset( $options[ $key ] ) ? ( selected( $options[ $key ], $value, false ) ) : ( '' ); ?>
            >
                <?php echo $position ?>
            </option>
        <?php } ?>
    </select>
    <?php
}

function peachpay_field_button_width_cb( $args ) {
    $options = get_option( 'peachpay_options' );
    $key = $args[ 'key' ];
    ?>
    <input
        id="peachpay_setting_<?php echo esc_attr( $key ); ?>"
        name='peachpay_options[<?php echo esc_attr( $key ); ?>]'
        type='text'
        value='<?php echo esc_attr( ($options && $options[ $key ]) ? $options[ $key ] : 220 ); ?>'
        style='width: 75px'
    /> px
    <?php
}

function peachpay_settings_init() {
    register_setting( 'peachpay', 'peachpay_options' );
    peachpay_settings_general();
    peachpay_settings_button();
}
add_action( 'admin_init', 'peachpay_settings_init' );

function peachpay_options_page() {
    add_menu_page(
        'PeachPay settings',
        'PeachPay',
        'read',
        'peachpay',
        'peachpay_options_page_html',
        'dashicons-cart',
        56
    );
}
add_action( "admin_menu", 'peachpay_options_page' );

function peachpay_options_page_html() {
    if ( ! current_user_can( 'read' ) ) {
        return;
    }

    // check if the user have submitted the settings
    // WordPress will add the "settings-updated" $_GET parameter to the url
    if ( isset( $_GET['settings-updated'] ) ) {
        $options = get_option( 'peachpay_options' );

        $args = array(
            'body' => array(
                'key' => $options['key'],
                'domain' => get_site_url()
            ),
            'httpversion' => '2.0',
            'blocking' => true
        );

        $response = wp_remote_post( peachpay_api_url() . 'auth', $args );

        if ( $response['response']['code'] === 200 ) {
            update_option( 'peachpay_valid_key', true );
            add_user_meta( get_current_user_id(), 'peachpay_notice_dismissed', 'true', true );
            add_settings_error( 'peachpay_messages', 'peachpay_message', __( 'Success! PeachPay is active and your settings have been saved.', 'peachpay' ), 'updated' );
        } else {
            update_option( 'peachpay_valid_key', false );
            if ( ! get_option( 'peachpay_options' )[ 'test_mode' ] ) {
                add_settings_error( 'peachpay_messages', 'peachpay_message', __( 'The key you entered is not valid. Please try again or contact us if you need help.', 'peachpay' ), 'error' );
            }
        }
    }

    // show error/success messages
    settings_errors( 'peachpay_messages' );
    ?>
    <div class="wrap">
        <h1><?php echo esc_html( get_admin_page_title() ); ?></h1>
        <form action="options.php" method="post">
            <?php
            // output security fields for the registered setting "peachpay"
            settings_fields( 'peachpay' );

            // output setting sections and their fields
            // (sections are registered for "peachpay", each field is registered to a specific section)
            do_settings_sections( 'peachpay' );

            // output save settings button
            submit_button( 'Save settings' );
            ?>
        </form>
    </div>
    <?php
}
