let variationID = '';
let uploadedFileURLs = [];
let loaded = false;
let buttonClickedBeforeLoad = false;
const peachpayVersion = php_data["version"];

window.onmessage = (event) => {
	if (event.data === 'openModal') {
		document.querySelector('#pp-modal-overlay').style.display = 'block';
		document.querySelector('body').style.overflow = 'hidden';
		window.scrollTo(0, 0);
		hideLoadingSpinner();
	}
	if (event.data === 'closeModal') {
		document.querySelector('#pp-modal-overlay').style.display = 'none';
		document.querySelector('body').style.overflow = 'auto';
	}
	if (event.data === 'loaded') {
		loaded = true;
		if (buttonClickedBeforeLoad) {
			sendButtonClickedMessage();
		}
	}
	if (event.data === 'cartReady') {
		// Need to retrigger this for special items with 30+ variations where the
		// variations load async and the first variation change message might arrive
		// before the init message
		if (variationID) {
			sendVariationChangedMessage(variationID);
		}
	}
	if (event.data.event === 'fetchCartFees') {
		fetchAndSendCartFees(event);
	}
	if (event.data.event === 'placeOrderDirectly') {
		placeOrder(event.data);
	}
}

function baseStoreURL() {
	return location.hostname === 'localhost'
	? `http://${location.hostname}:8000`
	: `https://${location.hostname}`;
}

function baseURL(merchantHostname) {
	if (php_data.test_mode) {
		if (merchantHostname === 'localhost' || merchantHostname === '127.0.0.1') {
			return `http://localhost:8080`;
		}
		return 'https://dev-connect.peachpaycheckout.com';
	}

	switch (merchantHostname) {
		case 'localhost':
		case '127.0.0.1':
			return `http://localhost:8080`;
		case 'woo.peachpay.app':
		case 'theme1.peachpay.app':
		case 'theme2.peachpay.app':
		case 'theme3.peachpay.app':
		case 'theme4.peachpay.app':
		case 'theme5.peachpay.app':
			return 'https://dev-connect.peachpaycheckout.com';
		default:
			return 'https://connect.peachpaycheckout.com';
	}
}

function basePeachPayAPIURL(merchantHostname) {
	if (php_data.test_mode) {
		return 'https://dev.peachpay.app';
	}
	switch (merchantHostname) {
		case 'localhost':
		case '127.0.0.1':
		case 'woo.peachpay.app':
		case 'theme1.peachpay.app':
		case 'theme2.peachpay.app':
		case 'theme3.peachpay.app':
		case 'theme4.peachpay.app':
		case 'theme5.peachpay.app':
			return 'https://dev.peachpay.app';
		default:
			return 'https://prod.peachpay.app';
	}
}

function init(options) {
	initContainer(options);
	initButton(options);
	sendInitMessage();

	const groupedQuantity = document.querySelector('form.cart.grouped_form');
	if (groupedQuantity) {
		php_data.cart.forEach(item => {
			const quantityForm = document.querySelector(`form.cart input[name="quantity[${item.product_id}]"]`);
			if (quantityForm) {
				quantityForm.addEventListener('change', () => sendGroupedQuantityChangedMessage(item.product_id));
			}
		});
	}

	const quantity = document.querySelector('form.cart input[name="quantity"]');
	if (!quantity || location.href.includes('cart')) {
		return;
	}
	quantity.addEventListener('change', sendQuantityChangedMessage);
}

function initButton(options) {
	const button = document.querySelector(`#pp-button${options.isMiniCart ? '-mini' : ''}`);

	// Apply some dynamic styles to the button
	button.style.width = options.width || '100%';
	button.style.setProperty('--button-color', php_data.button_color || '#ffa500');
	tryToMatchFontSize(button, options.isMiniCart);

	// It's easier to handle extra cart features such as coupons on the actual
	// cart page since the product pages were not exactly built for that, so
	// redirect to the cart page for these extra features
	if (!php_data.is_cart_page && (php_data.plugin_woocommerce_points_and_rewards_active || location.hostname === 'tires.pneupress.com')) {
		button.addEventListener('click', async () => {
			const success = await addCurrentProductToCart();
			if (success) {
				window.location = `${php_data.wc_cart_url}?open_peachpay`;
			}
		});
		return;
	}

	if (php_data.file_upload) {
		initFileUpload();
		document.querySelector(`#pp-button-text${options.isMiniCart ? '-mini' : ''}`).textContent = 'Choose a file';
		const uploader = document.querySelector('#uploader');
		button.addEventListener('click', () => {
			uploader.click();
		});
	} else {
		// most sites take this path
		button.addEventListener('click', event => openPeachPay(event));
	}

	// If coming from an extra feature redirect, open up PeachPay right away
	const urlParams = new URLSearchParams(location.search);
	if (urlParams.has('open_peachpay')) {
		openPeachPay();
	}
}

function tryToMatchFontSize(button, isMiniCart) {
	if (isMiniCart) {
		return;
	}
	let selector = '';
	if (php_data.is_cart_page) {
		selector = '.wc-proceed-to-checkout .button.checkout-button';
	} else {
		selector = 'button[name="add-to-cart"]';
	}
	const elementToCopy = document.querySelector(selector);
	if (elementToCopy) {
		button.style.fontSize = window.getComputedStyle(elementToCopy).fontSize;
	}
}

function openPeachPay(event) {
	let isMiniCart = false;

	if (event && event.currentTarget) {
		isMiniCart = event.currentTarget.id === 'pp-button-mini';
	}

	showLoadingSpinner(isMiniCart);

	if (!loaded) {
		buttonClickedBeforeLoad = true;
		return;
	}

	sendButtonClickedMessage(isMiniCart);
}

function initContainer(options) {
	if (options.isMiniCart) {
		// They cannot change the position of the button in the mini cart because
		// only the native style looks good
		return;
	}
	const container = document.querySelector('#pp-button-container');
	// On the product page, no position option is provided
	container.style.textAlign = options.position || 'inherit';
}

function initFileUpload() {
	document.querySelector('#pp-button-container').insertAdjacentHTML('afterbegin', uploadButton);
	document.querySelector('#uploader').addEventListener('change', async (event) => {
		const url = `${baseStoreURL()}/wp-json/peachpay/v1/upload`;
		uploadedFileURLs = JSON.parse(await sendFiles(url, event.target.files));
		showLoadingSpinner();
		sendButtonClickedMessage();
	});
}

const uploadButton = `<div id="upload-area">
<form>
	<input type="file" id="uploader" accept="image/*" name='files[]' multiple>
</form>
</div>`;

async function sendInitMessage() {
	const peachpay = document.querySelector('#peachpay-iframe');
	let merchantHostname = location.hostname;
	// Ideally we should source this from wp_options. It's gone through several
	// iterations, we have always chosen the wrong thing to derive the host from.
	// In this case, WP sites can be hosted in a subdirectory. This is just a
	// quick fix for one site.
	if (merchantHostname === 'www.brancalmelmelada.com') {
		merchantHostname += '/plataforma';
	}
	peachpay.addEventListener('load', async () => {
		peachpay.contentWindow.postMessage({
			event: 'init',
			merchantHostname,
			isTestMode: php_data.test_mode,
			isCartPage: php_data.is_cart_page,
			isGroupedProduct: isGroupedProduct(),
			isMobile: peachpay_isMobile(),
			phpData: php_data,
			quantity: isGroupedProduct() ? 0 : quantity(),
			overriddenCurrency: getOverriddenCurrency(),
			browserLocale: navigator.language || 'en-US',
		}, '*');
	});
}

// Compatibility for currency switcher plugins
function getOverriddenCurrency() {
	const alg = document.querySelector('#alg_currency_select');
	if (alg) {
		return alg.options[alg.selectedIndex].value;
	}
	const wcj = document.querySelector('#wcj-currency-select');
	if (wcj) {
		return wcj.options[wcj.selectedIndex].value;
	}
	return null;
}

async function sendButtonClickedMessage(isMiniCart) {
	if (isGroupedProduct() && !validGroupedQuantity()) {
		checkValidityOf('form.cart.grouped_form');
		return;
	}

	if (!isGroupedProduct() && !validQuantity()) {
		checkValidityOf('form.cart');
		return;
	}

	if (!isMiniCart) {
		triggerQuantityChange();
	}

	const peachpay = document.querySelector('#peachpay-iframe');
	const data = {
		event: 'buttonClicked',
		variationSelected: variationSelected(),
		inStock: inStock(parseInt(variationID)),
		quantity: isGroupedProduct() ? 0 : quantity(),
		uploadedFileURLs: JSON.stringify(uploadedFileURLs),
		isMiniCart,
	}
	if (!isMiniCart && !php_data.is_cart_page && php_data.plugin_woocommerce_product_addon) {
		addPPOMFieldsToProductMeta();
		// This is really only for the meta_data because at this point the checkout
		// window may have made some modifications to its own phpData.cart
		data.updatedCart = php_data.cart;
	}
	peachpay.contentWindow.postMessage(data, '*');
}

function checkValidityOf(selector) {
	const form = document.querySelector(selector);
	if (form) {
		form.reportValidity();
	}
	hideLoadingSpinner();
}

function triggerQuantityChange() {
	// Some themes somehow update the quantity without triggering change
	// or input events, so we need to trigger the event manually to make sure
	// we get the most recent quantity
	const input = document.querySelector('form.cart input[name="quantity"]');
	if (input) {
		input.dispatchEvent(new Event('change'));
	}

	const groupedQuantity = document.querySelector('form.cart.grouped_form');
	if (groupedQuantity) {
		php_data.cart.forEach(item => {
			const quantityForm = document.querySelector(`form.cart input[name="quantity[${item.product_id}]"]`);
			if (quantityForm) {
				quantityForm.dispatchEvent(new Event('change'));
			}
		});
	}
}

function addPPOMFieldsToProductMeta() {
	const inputs = document.querySelectorAll('input.ppom-input');
	if (inputs.length === 0) {
		return;
	}
	// These custom fields can only appear on a single product page, or at least
	// that's the assumption
	php_data.cart[0].meta_data = [];
	inputs.forEach(input => {
		php_data.cart[0].meta_data.push({
			key: input.id,
			value: input.value,
		});
	});
}

// Watch for when the variation changes so we can update our checkout data
(function customInputSetter() {
	const descriptor = Object.getOwnPropertyDescriptor(HTMLInputElement.prototype, 'value');
	const originalSet = descriptor.set;
	// define our own setter
	descriptor.set = function (val) {
		if (this.className === 'variation_id' && val !== '') {
			variationID = val;
			sendVariationChangedMessage(variationID);
		}
		originalSet.apply(this, arguments);
	}
	Object.defineProperty(HTMLInputElement.prototype, "value", descriptor);
})();

async function sendVariationChangedMessage(variationID) {
	const res = await fetch(`${baseStoreURL()}/wp-json/peachpay/v1/variation/details?id=${variationID}`);
	const details = await res.json();
	const peachpay = document.querySelector('#peachpay-iframe');
	peachpay.contentWindow.postMessage({
		event: 'variationChanged',
		productID: details.product_id,
		variation: parseInt(variationID),
		variationTitle: details.variation_title,
		price: details.price,
	}, '*');
}

function sendQuantityChangedMessage() {
	document.querySelector('#peachpay-iframe').contentWindow.postMessage({
		event: 'quantityChanged',
		quantity: quantity(),
	}, '*');
}

async function fetchAndSendCartFees(event) {
	const res = await fetch(`${basePeachPayAPIURL(location.hostname)}/api/v1/merchants/${location.hostname}/cart/fees`, {
		method: 'POST',
		headers: {'Content-Type': 'application/json'},
		body: JSON.stringify({
			items: event.data.cart,
			address: {
				country: event.data.address.country,
				state: event.data.address.state,
				city: event.data.address.city,
				postcode: event.data.address.postcode,
			}
		})
	});
	const peachpay = document.querySelector('#peachpay-iframe');
	peachpay.contentWindow.postMessage({
		event: 'cartFees',
		cartFees: await res.json(),
	}, '*');
}

async function placeOrder(data) {
	const formData = new FormData();
	formData.append('_wpnonce', php_data._wpnonce);

	const b = data.billingAddress;
	formData.append('billing_first_name', b.billing_first_name);
	formData.append('billing_last_name', b.billing_last_name);
	formData.append('billing_company', b.billing_company);
	formData.append('billing_email', b.billing_email);
	formData.append('billing_phone', b.billing_phone);
	formData.append('billing_country', b.billing_country);
	formData.append('billing_address_1', b.billing_address_1);
	formData.append('billing_address_2', b.billing_address_2);
	formData.append('billing_city', b.billing_city);
	formData.append('billing_state', b.billing_state);
	formData.append('billing_postcode', b.billing_postcode);

	const s = data.shippingAddress;
	formData.append('shipping_first_name', s.shipping_first_name);
	formData.append('shipping_last_name', s.shipping_last_name);
	formData.append('shipping_company', s.shipping_company);
	formData.append('shipping_country', s.shipping_country);
	formData.append('shipping_address_1', s.shipping_address_1);
	formData.append('shipping_address_2', s.shipping_address_2);
	formData.append('shipping_city', s.shipping_city);
	formData.append('shipping_state', s.shipping_state);
	formData.append('shipping_postcode', s.shipping_postcode);

	formData.append('shipping_method[0]', data.shippingMethod.method_id);
	formData.append('payment_method', 'peachpay');
	formData.append('payment_method_title', 'PeachPay');
	// Tentatively 1 for future separation of billing/shipping
	formData.append('ship_to_different_address', 1);
	formData.append('terms', 1);
	formData.append('european_gdpr', 1);

	const orderResponse = await fetch(`${baseStoreURL()}/?wc-ajax=wc_peachpay_create_order`, {
		method: 'POST',
		body: formData,
	});
	const orderResult = await orderResponse.json();

	if (orderResult.result === 'failure') {
		window.alert('Could not place order. Your payment may have already been accepted, so check before placing another order.');
		throw new Error(orderResult.messages);
	}

	// Orders placed this way still need to be recorded in our MongoDB database,
	// and we need to update the Stripe payment intent description with the order number,
	// so that's what this extra call is for
	await fetch(`${basePeachPayAPIURL(location.hostname)}/api/v1/session/order/finish`, {
		method: 'POST',
		headers: {'Content-Type': 'application/json'},
		body: JSON.stringify({
			sessionID: data.sessionID,
			orderID: orderResult.orderID
		})
	});

	// Redirect to the confirmation page
	window.location = orderResult.redirect;
}

async function addCurrentProductToCart() {
	const defaultID = document.querySelector('.single_add_to_cart_button').value;
	const productIDField = document.querySelector('input[name=product_id]');
	const quantityField = document.querySelector('input[name=quantity]');
	const variationIDField = document.querySelector('input[name=variation_id]');
	const data = new FormData();
	data.append('action', 'peachpay_woocommerce_ajax_add_to_cart');
	data.append('product_id', productIDField ? productIDField.value : defaultID);
	data.append('quantity', quantityField ? quantityField.value : 1);
	data.append('variation_id', variationIDField ? variationIDField.value : 0);
	const response = await fetch(wc_add_to_cart_params.ajax_url, {
		method: 'POST',
		body: data,
	});
	return response.status === 200;
}

function validQuantity() {
	const underMin = minQuantity() && (quantity() < minQuantity());
	const overMax = maxQuantity() && (quantity() > maxQuantity());
	return !underMin && !overMax;
}

function quantity() {
	const input = document.querySelector('form.cart input[name="quantity"]');
	return input ? input.value : 1;
}

function minQuantity() {
	const quantity = document.querySelector('form.cart input[name="quantity"]');
	return quantity ? parseInt(quantity.min) : false;
}

function maxQuantity() {
	const quantity = document.querySelector('form.cart input[name="quantity"]');
	return quantity ? parseInt(quantity.max) : false;
}

function sendGroupedQuantityChangedMessage(productID) {
	document.querySelector('#peachpay-iframe').contentWindow.postMessage({
		event: 'groupedQuantityChanged',
		quantity: groupedQuantity(productID),
		productID,
	}, '*');
}

function isGroupedProduct() {
	return document.querySelector('form.cart.grouped_form') !== null;
}

function validGroupedQuantity() {
	for (let i = 0; i < php_data.cart.length; i++) {
		const id = php_data.cart[i].product_id;
		const under = groupedQuantity(id) < minGroupedQuantity(id);
		const over = groupedQuantity(id) > maxGroupedQuantity(id);
		if (under || over) {
			return false;
		}
	}
	return true;
}

function groupedQuantity(productID) {
	const input = document.querySelector(`form.cart input[name="quantity[${productID}]"]`);
	return input ? input.value : 1;
}

function minGroupedQuantity(productID) {
	const quantity = document.querySelector(`form.cart input[name="quantity[${productID}]"]`);
	return quantity ? parseInt(quantity.min) : false;
}

function maxGroupedQuantity(productID) {
	const quantity = document.querySelector(`form.cart input[name="quantity[${productID}]"]`);
	return quantity ? parseInt(quantity.max) : false;
}

function variationSelected() {
	const variation = document.querySelector('input.variation_id');
	if (variation !== null && (variation.value === '0' || variation.value === '')) {
		hideLoadingSpinner();
		// A variation must be selected if given the option
		return false;
	}
	return true;
}

function inStock(id) {
	const variationForm = document.querySelector('form.variations_form');
	const variations = variationForm === null ? [] : JSON.parse(variationForm.dataset.product_variations);
	if (!variations) {
		// For items with 30+ style combinations, WC loads the style data when it's selected
		// instead of when the page loads. product_variations is "false" in this case. Will need
		// investigate how to check the stock. For now assume it's in stock.
		return true;
	}
	const variationMap = new Map();
	variations.forEach(v => variationMap.set(v.variation_id, v));

	// If variationForm is null and the product is out of stock, that means
	// we are on a simple product page, and in this state nothing shows up,
	// not even the PeachPay button.
	const inStock = variationForm === null || (variationMap.get(id) !== undefined && variationMap.get(id).is_in_stock);
	if (!inStock) {
		hideLoadingSpinner();
	}
	return inStock;
}

async function sendFiles(url, files) {
	const formData = new FormData();
	for (const name in files) {
		formData.append(`files[]`, files[name]);
	}
	const response = await fetch(url, {
		method: 'POST',
		body: formData,
	});
	return response.text();
}

function showLoadingSpinner(isMiniCart) {
	if (isMiniCart) {
		return;
	}
	document.querySelector('#loading-spinner').classList.remove('hide');
	document.querySelector('#pp-button-content').classList.add('hide');
}

function hideLoadingSpinner() {
	const loadingSpinner = document.querySelector('#loading-spinner');
	if (loadingSpinner) {
		loadingSpinner.classList.add('hide');
	}
	const buttonContent = document.querySelector('#pp-button-content');
	if (buttonContent) {
		buttonContent.classList.remove('hide');
	}
}

function spinnerURL() {
	// For now we explicity list out the sites that should have a dark loading
	// symbol because of their light button color
	const dark = location.hostname === 'www.blazecandles.co' ? '-dark' : '';
	return `${baseURL(location.hostname)}/img/spinner${dark}.svg`;
}

function peachpay_isMobile() {
	return /Mobi/.test(navigator.userAgent);
}

window.pp_peachpayButton = `
<div id="pp-button-container" class="pp-button-container ${(php_data.test_mode && !php_data.wp_admin_or_editor) ? 'hide' : ''}">
	<button id="pp-button" class="pp-button" type="button">
		<img src="${spinnerURL()}" id="loading-spinner" class="spinner hide">
		<div id="pp-button-content">
			<span id="pp-button-text">${pp_i18n['express-checkout'][php_data['language']]}</span>
		</div>
	</button>
</div>
`;

window.pp_peachpayButtonMiniCart = `
<div id="pp-button-container-mini" class="pp-button-container ${(php_data.test_mode && !php_data.wp_admin_or_editor) ? 'hide' : ''}">
	<button id="pp-button-mini" class="pp-button" type="button">
		<img src="${spinnerURL()}" id="loading-spinner-mini" class="spinner hide">
		<div id="pp-button-content-mini">
			<span id="pp-button-text-mini">${pp_i18n['express-checkout'][php_data['language']]}</span>
		</div>
	</button>
</div>
`;

window.pp_checkoutForm = `
<div id="pp-modal-overlay" class="pp-overlay">
	<iframe id="peachpay-iframe" src="${baseURL(location.hostname)}/peachpay-checkout.html">
		Unable to load PeachPay Checkout
	</iframe>
</div>

<style>
#peachpay-iframe {
	width: 100%;
	height: 100%;
	border: none;
}
.pp-overlay {
	background: rgba(40, 40, 40, 0.75);
	display: none;
	position: fixed;
	width: 100%;
	height: 100%;
	z-index: 9000000;
	overscroll-behavior: contain;
	top: 0; right: 0; bottom: 0; left: 0;
}
</style>
`;

// Per site customization to make the buttons look as native as possible

document.addEventListener('DOMContentLoaded', () => {
	peachpay_buttonShine();
	if (location.hostname === 'www.traveltek.com.au') {
		peachpay_traveltek();
	}
	if (location.hostname === 'katekos.com') {
		peachpay_katekos();
	}
	if (location.hostname === 'skregear.com') {
		peachpay_skregear();
	}
	if (location.hostname === 'waterluxe-osmosis.es') {
		document.querySelector('#pp-button-text').textContent = 'Compra ahora';
	}
	if (location.hostname === 'www.enotecacorsetti.com') {
		peachpay_enotecacorsetti();
	}
	if (location.hostname === 'agirlsbestfriend.ie') {
		peachpay_agirlsbestfriend();
	}
	if (location.hostname === 'salafibookstore.com') {
		peachpay_salafibookstore();
	}
});

function peachpay_buttonShine() {
	document.querySelector('body').insertAdjacentHTML('beforeend', peachpay_buttonShineCSS);
}

const peachpay_buttonShineCSS = `
<style>
@keyframes shine {
	100% {
		left: 200%;
	}
}
#pp-button:after,
#pp-button-mini:after {
	animation: shine 1.8s ease infinite;
	background: linear-gradient(to right, transparent, rgba(255, 255, 255, 0.3), transparent);
	content: '';
	display: inherit;
	height: 200%;
	left: -200%;
	position: absolute;
	top: 0;
	transform: skewX(-20deg);
	width: 50%;
}
</style>`;

function peachpay_traveltek() {
	const peachpayButton = document.querySelector('#pp-button');
	peachpayButton.style.cssText += ';font-family: -apple-system, system-ui, BlinkMacSystemFont, "Segoe UI", Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol" !important;';
	peachpayButton.style.cssText += ';text-transform: uppercase !important';
	peachpayButton.style.fontSize = '12px';
	if (php_data.is_cart_page) {
		peachpayButton.style.fontSize = '1.25em';
	}
}

function peachpay_katekos() {
	const peachpayButton = document.querySelector('#pp-button');
	peachpayButton.style.cssText += 'text-transform: uppercase !important;';
	peachpayButton.style.borderRadius = '10px';
	peachpayButton.style.fontSize = '20px';
	peachpayButton.style.cssText += ';font-family: Raleway !important;';
	peachpayButton.style.fontWeight = 400;
	peachpayButton.style.backgroundColor = '#050023';
	if (php_data.is_cart_page) {
		peachpayButton.style.padding = '10px 0';
	} else {
		peachpayButton.style.cssText += 'width: inherit !important;';
		peachpayButton.style.padding = '15px';
	}
}

function peachpay_skregear() {
	const peachpayButton = document.querySelector('#pp-button');
	peachpayButton.style.borderRadius = '0';
	peachpayButton.style.fontSize = '16px';
	peachpayButton.style.fontWeight = 700;
	peachpayButton.style.cssText += ';font-family: Lato !important;';
	peachpayButton.style.cssText += 'text-transform: uppercase !important;';
	if (php_data.is_cart_page) {
		peachpayButton.style.fontSize = '0.97em';
		peachpayButton.style.padding = '0.62em';
	}
}

function peachpay_enotecacorsetti() {
	const peachpayButton = document.querySelector('#pp-button');
	peachpayButton.style.borderRadius = '0';
	peachpayButton.style.backgroundColor = '#f15a21';
}

function peachpay_agirlsbestfriend() {
	const peachpayButton = document.querySelector('#pp-button');
	peachpayButton.style.borderRadius = '0';
}

function peachpay_salafibookstore() {
	const peachpayButton = document.querySelector('#pp-button');
	peachpayButton.style.borderRadius = '0';
}
